@@ -319,7 +319,7 @@ def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
     common_end(g,file_name_out,id2types)
 
 def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")],conserve_mem=False):
-    from limic.util import start, end, status, file_size, save_pickled, split
+    from limic.util import start, end, status, file_size, save_pickled, split, file_copy
     from xml.sax import parse
     from xml.sax.handler import ContentHandler
     from bz2 import open as bopen
@@ -327,17 +327,31 @@ def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]
     white = list(map(lambda x:(x[0],x[1][0]+'" v="'+x[1][1]+'"'),zip(range(len(white)),white)))
     black = list(map(lambda x:(-1,x[0]+'" v="'+x[1]+'"'),black))
     white_black = white+black
-    f = bopen(file_name_in,"rt") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rt")
     if not conserve_mem:
         start("Reading map data from",file_name_in)
-        data = f.read()
+        from mmap import mmap
+        area = mmap(-1,2**40)
+        if file_name_in[-4:] == ".bz2":
+            try:
+                from subprocess import Popen, PIPE
+                f = Popen(['lbzcat',file_name_in],stdout=PIPE).stdout
+            except Exception as e:
+                print(e)
+                status("WARNING: lbzcat failed - falling back on Python bz2 library")
+                f = bopen(file_name_in,"rb")
+        else:
+            f = open(file_name_in,"rb")
+        file_copy(f,area)
         f.close()
+        area_length = area.tell()
         end()
     start("Extracting ways for white and black")
     if conserve_mem:
+        f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
         elems = split(f,'\n  <')
     else:
-        elems = data.split('\n  <')
+        area.seek(0)
+        elems = split(area,'\n  <',maxlength=area_length)
     lines = []
     substations = []
     id2types = {}
@@ -366,8 +380,11 @@ def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]
             id2substations.setdefault(node,[]).append(substation)
     start("Extracting nodes for white and black")
     if conserve_mem:
-        f = bopen(file_name_in,"rt") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rt")
+        f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
         elems = split(f,'\n  <')
+    else:
+        area.seek(0)
+        elems = split(area,'\n  <',maxlength=area_length)
     towers = []
     nodes = []
     id2elem = {}
