@@ -4,11 +4,13 @@ NOGT = (("-g","--no-graph-tool"),{'action':'store_true','dest':'no_gt','default'
 URL = (("-d","--download-url"),{'type':str,'dest':'url','help':"define url for download directory to be URL",'metavar':'URL'})
 SHOW = (("-l","--list"),{'action':'store_true','dest':'show','default':False,'help':"list available countries (default: False)"})
 CONSERVEMEM = (("-c","--conserve-memory"),{'action':'store_true','dest':'conserve_mem','default':False,'help':"lower memory usage but higher runtime"})
+WORKERS = (("-w","--max-workers"),{'type':int,'dest':'max_workers','help':"set maximum number of parallel workers to WORKERS",'metavar':'WORKERS'})
+
 CONFIG = [
     ("stage0",{'help':"generate map data and all graph files (VERY SLOW)",'args':[SHOW,OVERPASS,NOGT,URL,COUNTRIES]}),
     ("stage1",{'help':"download map data and generate all graph files (SLOW)",'args':[
         ("cache",{'help':"cache files from Overpass API",'args':[SHOW,OVERPASS,NOGT,URL,COUNTRIES]}),
-        ("osm",{'help':"compressed XML from GeoFabrik",'args':[SHOW,CONSERVEMEM,NOGT,URL,COUNTRIES]})
+        ("osm",{'help':"compressed XML from GeoFabrik",'args':[SHOW,CONSERVEMEM,NOGT,WORKERS,URL,COUNTRIES]})
     ]}),
     ("stage2",{'help':"download NX graph files and other graph files",'args':[SHOW,NOGT,URL,COUNTRIES]}),
     ("stage3",{'help':"download all graph files (RECOMMENDED)",'args':[SHOW,NOGT,URL,COUNTRIES]})
@@ -27,12 +29,21 @@ def extract_cache_all(overpass_url,countries):
         file_name_out = "graph."+country+".nx"
         extract_cache(file_name_in,file_name_out,overpass_url,area=None,around=1000,eps=0.01,safe_dist=100,penalize=20)
 
-def extract_osm_all(countries,conserve_mem=False):
+def extract_osm_all(countries,conserve_mem=False,max_workers=None):
     from limic.extract import extract_osm
+    from concurrent.futures import ProcessPoolExecutor, wait
+    if max_workers:
+        executor = ProcessPoolExecutor(max_workers=max_workers)
+        fs = []
     for country in countries:
         file_name_in = country+"-latest.osm.bz2"
         file_name_out = "graph."+country+".nx"
-        extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,conserve_mem=conserve_mem)
+        if max_workers:
+            fs.append(executor.submit(extract_osm,file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,conserve_mem=conserve_mem))
+        else:
+            extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,conserve_mem=conserve_mem)
+    if max_workers:
+        wait(fs)
 
 def convert_nx_gt_all(countries):
     from limic.convert import convert_nx_gt
@@ -85,11 +96,17 @@ def init_stage1_cache(overpass_url,countries,no_gt,url=None,show=False):
     extract_cache_all(overpass_url,countries)
     convert_merge_all(countries,no_gt)
 
-def init_stage1_osm(countries,no_gt,url=None,show=False,conserve_mem=False):
+def init_stage1_osm(countries,no_gt,url=None,show=False,conserve_mem=False,max_workers=None):
     from limic.download import download_osm, common
-    download_osm(countries,url=url,show=show)
+    from limic.util import start, status
+    from os import cpu_count
+    if not max_workers:
+        max_workers = cpu_count()*4
+    start("Number of workers")
+    status(max_workers)
+    download_osm(countries,url=url,show=show,max_workers=max_workers)
     countries, url = common(countries,url,show,osm=True)
-    extract_osm_all(countries,conserve_mem=conserve_mem)
+    extract_osm_all(countries,conserve_mem=conserve_mem,max_workers=max_workers)
     convert_merge_all(countries,no_gt)
 
 def init_stage2(countries,no_gt,url=None,show=False):
