@@ -23,12 +23,13 @@ def serve_auto(countries,host="localhost",port=5000,prefix="",osm=False,url=None
     from limic.init import extract_osm_all, merge_all
     from limic.render import render_nx
     graph_file = None
-    countries, url = common(countries=countries,url=url,show=show,osm=osm)
     if osm:
+        countries, url = common(countries=countries,url=url,show=show,osm=osm,join=False)
         download_osm(countries=countries,url=url)
         extract_osm_all(countries=countries,conserve_mem=conserve_mem)
     else:
-        download_graph(suffix="nx",countries=countries,url=url,show=show)
+        countries, url = common(countries=countries,url=url,show=show,osm=osm,join=True)
+        download_graph(suffix="nx",countries=countries,url=url,show=show,join=True)
     if len(countries) > 1 :
         merge_all(countries)
     if len(countries) == 1:
