@@ -271,10 +271,15 @@ def save_path(costpath,out_file=None,visualize=False):
     else:
         file = open(out_file,"w") if out_file else stdout
         print("node(id:"+",".join(map(lambda x:str(x[3]),path))+");out;",file=file)
-def split(file,sep=None,maxsplit=-1,chunk=2**16):
+def split(file,sep=None,maxsplit=-1,chunk=2**16,maxlength=None):
     rest = ""
+    total = 0
     while True:
-        data = file.read(chunk)
+        data = file.read(chunk).decode('latin-1')
+        total += len(data)
+        if maxlength and total > maxlength:
+            data = data[:-(total-maxlength)]
+            chunk = 0
         if not data:
             break
         if rest:
@@ -284,3 +289,9 @@ def split(file,sep=None,maxsplit=-1,chunk=2**16):
         for part in parts:
             yield part
     yield rest
+def file_copy(f,g,chunk=2**16):
+    while True:
+        data = f.read(chunk)#.encode('utf8')
+        if not data:
+            break
+        g.write(data)
